
# React Native Smart Home Controller connector library

## Getting started

Packages hosted on GitLab need additional setup steps, before they can be installed:

1. Add the scope-level registry URLs to your project's `.npmrc` file.

```properties
@cs-bosch-emma:registry=https://src.codespring.ro/api/v4/projects/294/packages/npm/
```

2. You will need to use your [personal access token](https://docs.gitlab.com/ee/user/profile/personal_access_tokens.html#create-a-personal-access-token) with scope `api` for authentication.

```bash
npm config set -- '//src.codespring.ro/api/v4/projects/294/packages/npm/:_authToken' "<your_personal_access_token>"
```

Then, you can go on to install the dependency:

### yarn

```bash
yarn add @cs-bosch-emma/react-native-shc-connector
```

### npm

```bash
npm install @cs-bosch-emma/react-native-shc-connector --save
```

### Mostly automatic installation

On newer versions of React Native (0.61+) you are set!

For older versions:

```sh
react-native link @cs-bosch-emma/react-native-shc-connector
```

### Manual installation

#### iOS

1. In XCode, in the project navigator, right click `Libraries` ➜ `Add Files to [your project's name]`
2. Go to `node_modules` ➜ `@cs-bosch-emma/react-native-shc-connector` and add `RNShcConnector.xcodeproj`
3. In XCode, in the project navigator, select your project. Add `libRNShcConnector.a` to your project's `Build Phases` ➜ `Link Binary With Libraries`
4. Run your project (`Cmd+R`)<

#### Android

1. Open up `android/app/src/main/java/[...]/MainActivity.java`

  - Add `import com.bosch.tt.de.emma.connector.RNShcConnectorPackage;` to the imports at the top of the file
  - Add `new RNShcConnectorPackage()` to the list returned by the `getPackages()` method

2. Append the following lines to `android/settings.gradle`:

    ```groovy
    include ':react-native-shc-connector'
    project(':react-native-shc-connector').projectDir = new File(rootProject.projectDir, 	'../node_modules/@cs-bosch-emma/react-native-shc-connector/android')
    ```

3. Insert the following lines inside the dependencies block in `android/app/build.gradle`:

    ```groovy
    compile project(':react-native-shc-connector')
    ```

## Usage

```javascript
import RNShcConnector from '@cs-bosch-emma/react-native-shc-connector';

// Init module before using
RNShcConnector.initModule();

// Use the exported functions
RNShcConnector.connect();
```
