package com.bosch.tt.de.emma.connector.service;

import android.util.Log;

import androidx.annotation.NonNull;

import com.bosch.sh.connector.thirdparty.api.ThirdPartyConnectorLibrary;

import okhttp3.HttpUrl;
import okhttp3.MediaType;
import okhttp3.Request;
import okhttp3.RequestBody;

/**
 * Created by torokt on 03/11/2017.
 */
public class RequestService {
	private static final String TAG = RequestService.class.getSimpleName();
	private static final MediaType JSON = MediaType.parse("application/json; charset=utf-8");

	public static void get(@NonNull final String url,
						   @NonNull final ThirdPartyConnectorLibrary thirdPartyConnectorLibrary,
						   @NonNull final ShcCallback shcCallback) {

		Log.v(TAG, "Sending GET request to { " + url + " }");

		final Request.Builder requestBuilder = new Request.Builder().get();

		thirdPartyConnectorLibrary.httpClient().executeAsync(RequestService.createUrlBuilder(url), requestBuilder, new EmmaHttpResponseHandler(url, shcCallback));
	}

	public static void put(@NonNull final String url,
						   @NonNull final String jsonBody,
						   @NonNull final ThirdPartyConnectorLibrary thirdPartyConnectorLibrary,
						   @NonNull final ShcCallback shcCallback) {

		Log.v(TAG, "Sending PUT request to { " + url + " }");

		final Request.Builder requestBuilder = new Request.Builder().put(RequestService.createRequestBody(jsonBody));

		thirdPartyConnectorLibrary.httpClient().executeAsync(RequestService.createUrlBuilder(url), requestBuilder, new EmmaHttpResponseHandler(url, shcCallback));
	}

	public static void patch(@NonNull final String url,
							 @NonNull final String jsonBody,
							 @NonNull final ThirdPartyConnectorLibrary thirdPartyConnectorLibrary,
							 @NonNull final ShcCallback shcCallback) {

		Log.v(TAG, "Sending PATCH request to { " + url + " }");

		final Request.Builder requestBuilder = new Request.Builder().patch(RequestService.createRequestBody(jsonBody));

		thirdPartyConnectorLibrary.httpClient().executeAsync(RequestService.createUrlBuilder(url), requestBuilder, new EmmaHttpResponseHandler(url, shcCallback));
	}

	public static RequestBody createRequestBody(final String jsonBody) {
		RequestBody requestBody;

		if (jsonBody != null) {
			requestBody = RequestBody.create(jsonBody, JSON);
		} else {
			requestBody = RequestBody.create(new byte[]{}, null);
		}

		return requestBody;
	}

	public static HttpUrl.Builder createUrlBuilder(final String url) {
		String[] urlParts = url.split("\\?");

		final HttpUrl.Builder urlBuilder;

		//	This is a hack. Currently I didn't find any other solution. Maybe we should rewrite in the Javascript part 
		if (urlParts.length == 2) {
			urlBuilder = new HttpUrl.Builder()
				.addPathSegments(urlParts[0]).encodedQuery(urlParts[1]);
		} else {
			urlBuilder = new HttpUrl.Builder()
				.addPathSegments(urlParts[0]);
		}
		return urlBuilder;
	}
}
