"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTodayStyle = exports.getSelectedStyle = exports.getPeriodStyles = exports.baseCellStyle = exports.calendarStyle = exports.getCalendarTheme = exports.CALENDAR_TIME_FORMAT = void 0;
const react_native_1 = require("react-native");
const ui_1 = require("@cs-bosch-medusa/ui");
exports.CALENDAR_TIME_FORMAT = "YYYY-MM-DD";
const DEFAULT_FONT_SIZE = 16;
// NOTE: The lib uses hardcoded numbers internally as well, scaling values might break design
const CIRCLE_SIZE = 34;
const getCalendarTheme = (chartTheme, theme) => {
    const baseTheme = {
        backgroundColor: ui_1.Colors.TRANSPARENT,
        calendarBackground: theme.backgroundColor,
        dayTextColor: chartTheme.resolution.textColor,
        textDisabledColor: theme.disabledColor,
        todayTextColor: chartTheme.resolution.textColor,
        monthTextColor: chartTheme.resolution.textColor,
        textSectionTitleColor: chartTheme.resolution.textColor,
        selectedDayBackgroundColor: theme.primaryColor,
        selectedDayTextColor: chartTheme.resolution.selectedTextColor,
        textDayFontSize: DEFAULT_FONT_SIZE,
        textDayFontFamily: theme.regularFont,
        textDayFontWeight: "normal",
        textMonthFontFamily: theme.boldFont,
        textMonthFontSize: ui_1.FontSize.MEDIUM,
        textMonthFontWeight: react_native_1.Platform.OS === "ios" ? "700" : "600",
        textDayHeaderFontFamily: theme.regularFont,
        textDayHeaderFontWeight: "normal",
        textDayHeaderFontSize: DEFAULT_FONT_SIZE,
    };
    return {
        ...baseTheme,
        "stylesheet.calendar.header": {
            arrow: {
                padding: 0,
            },
            header: {
                flexDirection: "row",
                justifyContent: "space-between",
                paddingLeft: ui_1.METRICS.SCREEN.basePadding,
                paddingRight: ui_1.METRICS.SCREEN.basePadding,
                paddingBottom: 6,
                marginTop: 6,
                alignItems: "center",
            },
            week: {
                marginTop: ui_1.METRICS.CONTAINER.baseMargin,
                flexDirection: "row",
                justifyContent: "space-around",
                paddingHorizontal: ui_1.METRICS.CONTAINER.halfBasePadding,
            },
            dayHeader: {
                marginTop: 2,
                marginBottom: 7,
                width: CIRCLE_SIZE,
                textAlign: "center",
                fontSize: baseTheme.textDayHeaderFontSize,
                fontFamily: baseTheme.textDayHeaderFontFamily,
                fontWeight: baseTheme.textDayHeaderFontWeight,
                color: baseTheme.textSectionTitleColor,
            },
        },
        "stylesheet.calendar.main": {
            monthView: {
                backgroundColor: theme.backgroundColor,
                paddingHorizontal: ui_1.METRICS.CONTAINER.halfBasePadding,
            },
        },
    };
};
exports.getCalendarTheme = getCalendarTheme;
exports.calendarStyle = {
    // NOTE: paddingLeft and paddingRight are needed explicitly, paddingHorizontal won't work
    paddingLeft: 0,
    paddingRight: 0,
};
exports.baseCellStyle = {
    width: CIRCLE_SIZE,
    height: CIRCLE_SIZE,
    alignItems: "center",
    justifyContent: "center",
};
const getPeriodStyles = (chartTheme, theme) => ({
    fillers: {
        position: "absolute",
        height: CIRCLE_SIZE,
        flexDirection: "row",
        left: 0,
        right: 0,
    },
    leftFiller: {
        height: CIRCLE_SIZE,
        flex: 1,
    },
    rightFiller: {
        height: CIRCLE_SIZE,
        flex: 1,
    },
    wrapper: {
        alignItems: "center",
        alignSelf: "stretch",
    },
    text: {
        marginTop: react_native_1.Platform.OS === "android" ? 4 : 6,
        fontSize: DEFAULT_FONT_SIZE,
        fontFamily: theme.regularFont,
        color: chartTheme.resolution.textColor,
        backgroundColor: ui_1.Colors.TRANSPARENT,
    },
});
exports.getPeriodStyles = getPeriodStyles;
const getSelectedStyle = (theme) => ({
    backgroundColor: theme.resolution.selectedColor,
    borderRadius: CIRCLE_SIZE / 2,
});
exports.getSelectedStyle = getSelectedStyle;
const getTodayStyle = (theme) => ({
    borderRadius: CIRCLE_SIZE / 2,
    borderColor: theme.resolution.textColor,
    borderWidth: 1,
});
exports.getTodayStyle = getTodayStyle;
