import React, { FC, ReactNode } from "react";
import { ViewStyle, StyleProp } from "react-native";
import { ButtonIconPosition, ButtonVariant } from "./button-enums";
import { IconProps, IconSize } from "../icons";
import { TestProps } from "../../util";
export type ButtonProps = TestProps & {
    /** If true, disable all interactions for this component. */
    disabled?: boolean;
    /** The variant of button to be used. Defaults to `ButtonVariant.PRIMARY`. */
    variant?: ButtonVariant;
    /**
     * The text to display on the button.
     *
     * If neither the text nor an icon specified, the button renders the children it receives.
     */
    text?: string;
    /**
     * A medusa-compatible icon component to show in the button.
     * If text is also present, the icon is shown to the left of the text.
     *
     * If neither the text nor an icon specified, the button renders the children it receives.
     */
    icon?: React.ComponentType<IconProps>;
    /** The size of the `icon`. Defaults to `SMALL`. */
    iconSize?: IconSize;
    /**
     * Where to position the icon inside the button.
     *
     * Defaults to `BUTTON_LEFT`.
     */
    iconPosition?: ButtonIconPosition;
    /** Additional styling rules for the component. */
    style?: StyleProp<ViewStyle>;
    /** Callback that is called when the button is pressed. */
    onPress?(): void;
    /**
     * Components to display.
     */
    children?: ReactNode;
};
/**
 * Button component with theme colors and Bosch design.
 * Supports three variants of buttons: `ButtonVariant.PRIMARY` (default),
 * `ButtonVariant.SECONDARY` and `ButtonVariant.TERTIARY`.
 * If no `text` property is specified, the component renders the children passed to it.
 */
export declare const Button: FC<ButtonProps>;
//# sourceMappingURL=button.d.ts.map