"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Button = void 0;
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const text_1 = require("../text");
const button_enums_1 = require("./button-enums");
const icons_1 = require("../icons");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    button: {
        alignItems: "center",
        borderWidth: 1,
        flex: 1,
        flexDirection: "row",
        justifyContent: "center",
        maxHeight: theme_1.METRICS.BUTTON.height,
        minHeight: theme_1.METRICS.BUTTON.height,
        padding: theme_1.METRICS.CONTAINER.basePadding,
    },
    floatingIcon: {
        left: theme_1.METRICS.CONTAINER.halfBasePadding,
        position: "absolute",
    },
    spacedText: {
        marginLeft: theme_1.METRICS.CONTAINER.halfBaseMargin,
    },
});
/**
 * Button component with theme colors and Bosch design.
 * Supports three variants of buttons: `ButtonVariant.PRIMARY` (default),
 * `ButtonVariant.SECONDARY` and `ButtonVariant.TERTIARY`.
 * If no `text` property is specified, the component renders the children passed to it.
 */
const Button = ({ disabled = false, onPress, text, icon: Icon, iconSize = icons_1.IconSize.SMALL, iconPosition = button_enums_1.ButtonIconPosition.BUTTON_LEFT, variant = button_enums_1.ButtonVariant.PRIMARY, style, children, testID, }) => {
    const [pressed, setPressed] = (0, react_1.useState)(false);
    const { theme } = (0, theme_1.useTheme)();
    const backgroundColor = getBackgroundColor(variant, disabled, pressed, theme);
    const borderColor = getBorderColor(variant, disabled, pressed, theme);
    const borderWidth = getBorderWidth(variant);
    const color = getForegroundColor(variant, disabled, pressed, theme);
    const buttonStyle = react_native_1.StyleSheet.flatten([
        styles.button,
        { backgroundColor, borderColor, borderWidth },
        style,
    ]);
    const hasContent = Icon !== undefined || text !== undefined;
    const onPressIn = (0, react_1.useCallback)(() => setPressed(true), []);
    const onPressOut = (0, react_1.useCallback)(() => setPressed(false), []);
    const textSpacingForIcon = Icon
        ? iconPosition === button_enums_1.ButtonIconPosition.BUTTON_LEFT
            ? { paddingHorizontal: iconSize }
            : styles.spacedText
        : undefined;
    return (react_1.default.createElement(react_native_1.TouchableWithoutFeedback, { disabled: disabled, onPress: onPress, onPressIn: onPressIn, onPressOut: onPressOut, ...(0, util_1.getTestIdProps)(testID) },
        react_1.default.createElement(react_native_1.View, { style: buttonStyle },
            Icon &&
                (iconPosition === button_enums_1.ButtonIconPosition.BUTTON_LEFT ? (react_1.default.createElement(react_native_1.View, { style: text ? styles.floatingIcon : undefined },
                    react_1.default.createElement(Icon, { fillColor: color, iconSize: iconSize }))) : (react_1.default.createElement(Icon, { fillColor: color, iconSize: iconSize }))),
            text !== undefined && (react_1.default.createElement(text_1.Label, { style: [
                    textSpacingForIcon,
                    {
                        color,
                    },
                ] }, text)),
            !hasContent && children)));
};
exports.Button = Button;
const getBackgroundColor = (buttonVariant, disabled, pressed, theme) => {
    switch (buttonVariant) {
        case button_enums_1.ButtonVariant.PRIMARY:
            if (disabled) {
                return theme.disabledColor;
            }
            if (pressed) {
                return theme.pressedColor;
            }
            return theme.primaryColor;
        case button_enums_1.ButtonVariant.SECONDARY:
        case button_enums_1.ButtonVariant.TERTIARY:
            if (pressed) {
                return theme.secondaryPressedColor;
            }
            return theme.backgroundColor;
    }
};
const getBorderColor = (buttonVariant, disabled, pressed, theme) => {
    switch (buttonVariant) {
        case button_enums_1.ButtonVariant.PRIMARY:
            if (pressed) {
                return theme.pressedColor;
            }
            return theme.primaryColor;
        case button_enums_1.ButtonVariant.SECONDARY:
            if (disabled) {
                return theme.disabledColor;
            }
            if (pressed) {
                return theme.pressedColor;
            }
            return theme.primaryColor;
        case button_enums_1.ButtonVariant.TERTIARY:
            return theme_1.Colors.TRANSPARENT;
    }
};
const getBorderWidth = (buttonVariant) => {
    switch (buttonVariant) {
        case button_enums_1.ButtonVariant.PRIMARY:
        case button_enums_1.ButtonVariant.TERTIARY:
            return 0;
        case button_enums_1.ButtonVariant.SECONDARY:
            return 1;
    }
};
const getForegroundColor = (buttonVariant, disabled, pressed, theme) => {
    switch (buttonVariant) {
        case button_enums_1.ButtonVariant.PRIMARY:
            if (disabled) {
                return theme.disabledTextColor;
            }
            return theme.secondaryTextColor;
        case button_enums_1.ButtonVariant.SECONDARY:
        case button_enums_1.ButtonVariant.TERTIARY:
            if (disabled) {
                return theme.disabledColor;
            }
            if (pressed) {
                return theme.pressedColor;
            }
            return theme.primaryColor;
    }
};
