import { FC, ReactNode } from "react";
import { ViewStyle, StyleProp } from "react-native";
export type KeyboardContainerProps = {
    /** ViewStyle of the container */
    style?: StyleProp<ViewStyle>;
    /**
     * The distance between the top of the screen and the container.
     * This distance is used to properly scroll the screen to the right
     * position when the keyboard appears.
     */
    headerHeight?: number;
    /**
     * Components to display.
     */
    children?: ReactNode;
};
/**
 * A container component which resolves the input field covered by the Keyboard issue on iOS.
 *
 * If the keyboard covers an input field on Android then check if the `windowSoftInputMode`
 * is properly set in the `AndroidManifest.xml` file. Android documentation can be found
 * [here](https://developer.android.com/guide/topics/manifest/activity-element#wsoft).
 *
 * If the keyboard covers the selected input field on iOS then check if the `headerHeight`
 * property is correctly set to the distance between the top of the screen and the
 * container and make sure that the children are nested into scrollable component.
 */
export declare const KeyboardContainer: FC<KeyboardContainerProps>;
//# sourceMappingURL=keyboard-container.d.ts.map