"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomDropdown = void 0;
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const icons_1 = require("../icons");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const dropdown_animation_hook_1 = require("./dropdown-animation-hook");
const HEADER_HEIGHT = (0, theme_1.verticalScale)(50);
const styles = react_native_1.StyleSheet.create({
    collapsibleView: {
        overflow: "hidden",
    },
    dropdownHeader: {
        alignItems: "center",
        flexDirection: "row",
        maxHeight: HEADER_HEIGHT,
        minHeight: HEADER_HEIGHT,
        paddingLeft: theme_1.METRICS.SCREEN.basePadding,
        paddingRight: theme_1.METRICS.CONTAINER.basePadding,
        paddingVertical: theme_1.METRICS.CONTAINER.halfBasePadding,
    },
    headerComponent: {
        flexGrow: 1,
        flexShrink: 1,
    },
    modalContainer: {
        flex: 1,
    },
    openIcon: {
        alignItems: "center",
        padding: theme_1.METRICS.ICON.basePadding,
    },
    shadow: {
        elevation: 5,
        shadowColor: theme_1.Colors.BLACK,
        shadowOffset: {
            width: 0,
            height: 0,
        },
        shadowOpacity: 0.3,
        shadowRadius: 3,
    },
});
const isDropdownItem = (child) => {
    return typeof child === "object" && "props" in (child !== null && child !== void 0 ? child : {});
};
const CustomDropdown = (props) => {
    const { value: currentValue, disabled = false, onOptionPress, testID, children, screenInFocus, headerComponent, } = props;
    const { theme } = (0, theme_1.useTheme)();
    // A non-zero initial value is needed so onLayout can start measuring height dynamically.
    // Otherwise, it just stays at 0.
    const [optionsHeight, setOptionsHeight] = (0, react_1.useState)(1);
    const { OpenIcon, toggle, dropdownHeaderRef, isOpen, animatedViewStyle, justifyContent, } = (0, dropdown_animation_hook_1.useDropdownAnimation)(optionsHeight);
    const childrenArray = children
        ? react_1.default.Children.toArray(children).filter(isDropdownItem)
        : undefined;
    const onOptionsLayout = (0, react_1.useCallback)((event) => {
        const roundedHeight = Math.round(event.nativeEvent.layout.height);
        setOptionsHeight(roundedHeight);
    }, []);
    const onPress = (0, react_1.useCallback)((value) => {
        onOptionPress(value);
        toggle();
    }, [onOptionPress, toggle]);
    const disableInteractions = disabled || !childrenArray || childrenArray.length === 0;
    const fillColor = disabled ? theme.dropdown.disabledColor : theme.textColor;
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(react_native_1.Pressable, { disabled: disableInteractions, onPress: toggle, ref: dropdownHeaderRef, style: [
                styles.dropdownHeader,
                {
                    backgroundColor: theme.dropdown.backgroundColor,
                },
            ], ...(0, util_1.getTestIdProps)(`${testID}.toggle`) },
            react_1.default.createElement(react_native_1.View, { style: styles.headerComponent }, headerComponent),
            react_1.default.createElement(react_native_1.View, { style: styles.openIcon },
                react_1.default.createElement(OpenIcon, { fillColor: fillColor, iconSize: icons_1.IconSize.XSMALL }))),
        react_1.default.createElement(react_native_1.Modal, { onRequestClose: !disableInteractions ? toggle : undefined, transparent: true, visible: isOpen && screenInFocus, ...(0, util_1.getTestIdProps)(`${testID}.modal`) },
            react_1.default.createElement(react_native_1.Pressable, { accessible: false, disabled: disabled, onPress: toggle, style: styles.modalContainer, ...(0, util_1.getTestIdProps)(`${testID}.overlay`) },
                react_1.default.createElement(react_native_1.Animated.View, { style: [
                        styles.collapsibleView,
                        animatedViewStyle,
                        { justifyContent },
                    ] },
                    react_1.default.createElement(react_native_1.View, { onLayout: onOptionsLayout, style: [
                            styles.shadow,
                            {
                                // Note: bg color needed so elevation works on android.
                                // It is painted over by the list items anyways, so the color
                                // is not all that important.
                                // See: https://stackoverflow.com/a/27518160/6932518
                                backgroundColor: theme.dropdown.backgroundColor,
                                minHeight: optionsHeight,
                            },
                        ] }, childrenArray === null || childrenArray === void 0 ? void 0 : childrenArray.map((item, index) => (react_1.default.createElement(DropdownItemWrapper, { currentValue: currentValue, item: item, 
                        // eslint-disable-next-line react/no-array-index-key
                        key: index, onPress: onPress, testID: `${testID}.options.${index}` })))))))));
};
exports.CustomDropdown = CustomDropdown;
const DropdownItemWrapper = ({ currentValue, item, onPress, testID, }) => {
    var _a;
    return react_1.default.cloneElement(item, {
        currentValue,
        onPress,
        testID: (_a = item.props.testID) !== null && _a !== void 0 ? _a : testID,
    });
};
