import React from "react";
import { StyleProp, TextStyle } from "react-native";
import { Colors } from "../../theme";
import { TestProps } from "../../util";
import { TextFieldRef } from "./text-field-ref";
type TextFieldType = "text" | "password";
type TextFieldAutoCapitalize = "sentences" | "none" | "words";
type TextFieldKeyboardType = "default" | "number-pad" | "decimal-pad" | "numeric" | "email-address" | "phone-pad";
type TextFieldTextAlign = "left" | "right" | "center";
export type TextFieldPointerEvents = "box-none" | "none" | "box-only" | "auto";
export type InputFieldProps = TestProps & {
    /** Style of the text field. */
    style?: StyleProp<TextStyle>;
    /** Current value of the text field. Provide this if you want a controlled component. */
    value?: string;
    /**
     * Whether interactions are disabled for the text field.
     * The colors change to the theme's disabled color if it is true.
     *
     * Defaults to `false`.
     */
    disabled?: boolean;
    /** Limits the maximum number of characters that can be entered. */
    maxLength?: number;
    /** The type of text field to render. Defaults to `text`. */
    type?: TextFieldType;
    /** Placeholder for the text field. This text will be rendered before text input has been entered. */
    placeholder?: string;
    /**
     * Color of the shown placeholder text. Default color is from theme.
     * If you override it from outside, take care to handle different states (default, enabled, etc.).
     */
    placeholderColor?: Colors;
    /** Automatically capitalize certain characters. Defaults to `sentences`. */
    autoCapitalize?: TextFieldAutoCapitalize;
    /** Keyboard type which will appear when the text field is selected. Defaults to `default`. */
    keyboardType?: TextFieldKeyboardType;
    /** Where to align the text inside the input. */
    textAlign?: TextFieldTextAlign;
    /** Callback that is called when the value in the text field changes. */
    onValueChange: (value: string) => void;
    /** Callback that is called when the text field gets focused. */
    onFocus?: () => void;
    /** Callback that is called when the input gets blurred. */
    onBlur?: () => void;
};
export declare const InputField: React.ForwardRefExoticComponent<Omit<InputFieldProps, "keyboardType"> & {
    /** Keyboard type which will appear when the text field is selected. Defaults to `default`. */
    keyboardType?: TextFieldKeyboardType | "numbers-and-punctuation" | undefined;
    /**
     * Whether the contents of the text fields can be changed.
     * Defaults to `true`.
     */
    editable?: boolean | undefined;
    /** Controls whether the component can be the target of touch events. */
    pointerEvents?: TextFieldPointerEvents | undefined;
} & React.RefAttributes<TextFieldRef>>;
export {};
//# sourceMappingURL=input-field.d.ts.map