"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextFieldValidationInfo = exports.ValidationResult = void 0;
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const text_field_style_1 = require("./text-field-style");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    message: {
        fontSize: theme_1.FontSize.TINY,
        paddingVertical: 3,
        position: "absolute",
    },
    noOverflow: {
        overflow: "hidden",
    },
    separator: {
        width: "100%",
    },
});
var ValidationResult;
(function (ValidationResult) {
    ValidationResult[ValidationResult["NONE"] = 0] = "NONE";
    ValidationResult[ValidationResult["OK"] = 1] = "OK";
    ValidationResult[ValidationResult["WARNING"] = 2] = "WARNING";
    ValidationResult[ValidationResult["ERROR"] = 3] = "ERROR";
})(ValidationResult = exports.ValidationResult || (exports.ValidationResult = {}));
const usePrevious = (value) => {
    const ref = (0, react_1.useRef)();
    (0, react_1.useEffect)(() => {
        ref.current = { ...value };
    }, [value]);
    return ref.current;
};
const TextFieldValidationInfo = ({ validationResult = ValidationResult.NONE, focused, disabled = false, testID, validationMessage, }) => {
    var _a, _b;
    const { theme } = (0, theme_1.useTheme)();
    const animatedColor = (0, react_1.useRef)(new react_native_1.Animated.Value(0));
    const animatedTextHeight = (0, react_1.useRef)(new react_native_1.Animated.Value(0));
    const [textHeight, setTextHeight] = (0, react_1.useState)(0);
    const colors = useValidationInfoColors({
        focused,
        disabled,
        validationResult,
    });
    const previousColor = usePrevious(colors);
    const displayedMessage = validationResult !== ValidationResult.NONE
        ? validationMessage
        : undefined;
    // Animate collapsing
    (0, react_1.useEffect)(() => {
        const wasCollapsed = !disabled && textHeight > 0 && displayedMessage;
        react_native_1.Animated.timing(animatedTextHeight.current, {
            duration: theme_1.ANIMATION_DURATION.COLLAPSE,
            toValue: wasCollapsed ? textHeight : 0,
            useNativeDriver: false,
        }).start(() => {
            if (disabled || !displayedMessage) {
                /**
                 * We need to set textHeight to 0 after the animation is done,
                 * because onLayout is not called since the component is not mounted anymore
                 */
                setTextHeight(0);
            }
        });
    }, [disabled, textHeight, displayedMessage]);
    // Animate color and text change
    (0, react_1.useEffect)(() => {
        // Set to 0 to always interpolate a change.
        animatedColor.current.setValue(0);
        react_native_1.Animated.timing(animatedColor.current, {
            duration: theme_1.ANIMATION_DURATION.FADE,
            toValue: 1,
            useNativeDriver: false,
        }).start();
    }, [disabled, validationResult, displayedMessage]);
    const onLayout = (0, react_1.useCallback)((event) => {
        setTextHeight(event.nativeEvent.layout.height);
    }, []);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(react_native_1.Animated.View, { style: [
                styles.separator,
                {
                    backgroundColor: animatedColor.current.interpolate({
                        inputRange: [0, 1],
                        outputRange: [
                            (_a = previousColor === null || previousColor === void 0 ? void 0 : previousColor.lineColor) !== null && _a !== void 0 ? _a : theme.separatorColor,
                            colors.lineColor,
                        ],
                    }),
                    height: theme.separatorSize,
                },
            ], ...(0, util_1.getTestIdProps)(`${testID}.separator`) }),
        react_1.default.createElement(react_native_1.Animated.View, { style: [
                styles.noOverflow,
                {
                    height: animatedTextHeight.current,
                    backgroundColor: animatedColor.current.interpolate({
                        inputRange: [0, 1],
                        outputRange: [
                            (_b = previousColor === null || previousColor === void 0 ? void 0 : previousColor.fillColor) !== null && _b !== void 0 ? _b : theme.backgroundColor,
                            colors.fillColor,
                        ],
                    }),
                },
            ] }, !disabled && displayedMessage ? (react_1.default.createElement(react_native_1.Animated.Text, { allowFontScaling: theme.allowFontScaling, onLayout: onLayout, style: [
                text_field_style_1.textFieldStyle.textContainer,
                styles.message,
                {
                    fontFamily: theme.regularFont,
                    color: colors.textColor,
                },
            ], ...(0, util_1.getTestIdProps)(`${testID}.message`) }, displayedMessage)) : undefined)));
};
exports.TextFieldValidationInfo = TextFieldValidationInfo;
const useValidationInfoColors = ({ focused, disabled, validationResult, }) => {
    const { theme } = (0, theme_1.useTheme)();
    if (disabled) {
        return theme.inputs.disabled;
    }
    else {
        switch (validationResult) {
            case ValidationResult.OK:
                return theme.success;
            case ValidationResult.WARNING:
                return theme.warning;
            case ValidationResult.ERROR:
                return theme.error;
            default:
                return focused ? theme.inputs.focused : theme.inputs.default;
        }
    }
};
