import { ReactNode } from "react";
import { Colors } from "../../theme";
import { TestProps } from "../../util";
type BaseListItemProps = TestProps & {
    /** Main text of the list item. */
    text?: string;
    /** A smaller text to display below the main text line. */
    description?: string;
    /** Status text to show on the right side of the list item. */
    status?: string;
    /** If true, disables interaction with the item. Default is false. */
    disabled?: boolean;
    /**
     * Custom component to render at the left edge of the item.
     * Used mostly for displaying icons. Works best with `IconSize.SMALL`.
     */
    leftComponent?: JSX.Element;
    /**
     * Custom component to render in the center of the item.
     * The `text` and `description` properties will be ignored
     * if `middleComponent` is specified.
     */
    middleComponent?: JSX.Element;
    /**
     * Whether or not to render a separator at the bottom of the item.
     * Default is true.
     */
    separator?: boolean;
    /**
     * Sets the color of the text, description, status and arrow to this property.
     * If the list item is disabled then the disabled text color will be applied.
     */
    textColor?: Colors;
    /**
     * Defines the size of the vertical padding of the component. Possible values are NORMAL and LARGE.
     * Defaults to NORMAL.
     */
    spacing?: ListItemSpacing;
};
export declare enum ListItemSpacing {
    NORMAL = 0,
    LARGE = 1
}
export type ListItemProps = BaseListItemProps & {
    /**
     * Whether or not to render a right-facing arrow at the right edge of the item.
     * Default is false.
     */
    arrow?: boolean;
    /**
     * Custom component to render at the right edge of the item.
     * The arrow (if present) is displayed between this and the right edge.
     *
     * Used mostly for displaying icons, switches, checkboxes and other input components.
     * Works best with `IconSize.SMALL`.
     */
    rightComponent?: JSX.Element;
    /**
     * Callback to be called when list item is pressed.
     * The entire list item becomes touchable when this callback is present.
     */
    onPress?(): void;
};
export type CollapsibleListItemProps = BaseListItemProps & {
    /**
     * Components to display when the list item is not collapsed.
     */
    children: ReactNode;
};
export {};
//# sourceMappingURL=list-item-props.d.ts.map