"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useNotification = void 0;
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const notification_1 = require("./notification");
const DEFAULT_NOTIFICATION_DURATION_SECONDS = 5;
const styles = react_native_1.StyleSheet.create({
    container: {
        position: "absolute",
        top: 0,
        width: "100%",
        zIndex: 999,
    },
});
/**
 * Returns an object with a constant reference that contains the Notification
 * component as well as the show and hide methods.
 */
const useNotification = () => {
    const { theme } = (0, theme_1.useTheme)();
    const [visible, setVisible] = (0, react_1.useState)(false);
    const isMounted = (0, react_1.useRef)(true);
    const opacity = (0, react_1.useRef)(new react_native_1.Animated.Value(0));
    const timeout = (0, react_1.useRef)();
    (0, react_1.useEffect)(() => {
        isMounted.current = true;
        return () => {
            isMounted.current = false;
        };
    }, []);
    // Make sure the returned reference doesn't change between renders.
    const returnObject = (0, react_1.useMemo)(() => {
        // NOTE: We need this inside useMemo to avoid having to ensure referential equality
        // eslint-disable-next-line unicorn/consistent-function-scoping
        const clearCloseTimeout = () => {
            if (timeout.current !== undefined) {
                clearTimeout(timeout.current);
                timeout.current = undefined;
            }
        };
        const show = (config = {}) => {
            const { duration = DEFAULT_NOTIFICATION_DURATION_SECONDS, autoHide = true, } = config;
            clearCloseTimeout();
            setVisible(true);
            react_native_1.Animated.timing(opacity.current, {
                toValue: 1,
                duration: theme_1.ANIMATION_DURATION.FADE,
                useNativeDriver: true,
            }).start();
            if (autoHide && duration > 0) {
                timeout.current = setTimeout(() => {
                    // Avoid setting state of unmounted component
                    if (isMounted.current) {
                        hide();
                    }
                }, duration * 1000);
            }
        };
        const hide = () => {
            clearCloseTimeout();
            react_native_1.Animated.timing(opacity.current, {
                toValue: 0,
                duration: theme_1.ANIMATION_DURATION.FADE,
                useNativeDriver: true,
            }).start(() => setVisible(false));
        };
        return {
            show,
            hide,
        };
    }, []);
    returnObject.Notification = (props) => visible ? (react_1.default.createElement(react_native_1.Animated.View, { style: [
            styles.container,
            {
                backgroundColor: theme.backgroundColor,
                opacity: opacity.current,
            },
        ] },
        react_1.default.createElement(notification_1.Notification, { onDismiss: returnObject.hide, ...props }))) : // eslint-disable-next-line unicorn/no-null
        null;
    returnObject.visible = visible;
    return returnObject;
};
exports.useNotification = useNotification;
