"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Switch = void 0;
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const icons_1 = require("../icons");
const styles = react_native_1.StyleSheet.create({
    center: {
        borderRadius: icons_1.IconSize.XSMALL / 4,
        height: icons_1.IconSize.XSMALL / 2,
        width: icons_1.IconSize.XSMALL / 2,
    },
    switch: {
        borderRadius: icons_1.IconSize.XSMALL / 2,
        height: icons_1.IconSize.XSMALL,
        width: 2 * icons_1.IconSize.XSMALL,
    },
    switchWrapper: {
        height: icons_1.IconSize.XSMALL,
        width: 2 * icons_1.IconSize.XSMALL,
    },
});
const Switch = ({ value, onValueChange, disabled = false, style, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const animatedPosition = (0, react_1.useRef)(new react_native_1.Animated.Value(0));
    const animatedColor = (0, react_1.useRef)(new react_native_1.Animated.Value(0));
    const prevValue = (0, react_1.useRef)(value);
    (0, react_1.useEffect)(() => {
        react_native_1.Animated.spring(animatedPosition.current, {
            toValue: value ? 1 : 0,
            useNativeDriver: false,
        }).start();
        // on a change always trigger a transition
        animatedColor.current.setValue(0);
        react_native_1.Animated.timing(animatedColor.current, {
            toValue: 1,
            useNativeDriver: false,
            duration: 300,
        }).start(() => {
            prevValue.current = value;
        });
    }, [value]);
    const onSwitchPressed = (0, react_1.useCallback)(() => {
        onValueChange(!value);
    }, [onValueChange, value]);
    return (react_1.default.createElement(react_native_1.Pressable, { disabled: disabled, onPress: onSwitchPressed, style: [styles.switchWrapper, style], ...(0, util_1.getTestIdProps)(testID) }, ({ pressed }) => {
        const { switchStyle, circleStyle } = getAnimatedSwitchStyles(theme, animatedColor, animatedPosition, {
            pressed,
            on: value,
            prevValue: prevValue.current,
            disabled,
        });
        return (react_1.default.createElement(react_native_1.Animated.View, { style: [
                styles.switch,
                {
                    borderWidth: theme.switch.borderWidth,
                    padding: icons_1.IconSize.XSMALL / 4 -
                        theme.switch.borderWidth,
                },
                switchStyle,
            ] },
            react_1.default.createElement(react_native_1.Animated.View, { style: [styles.center, circleStyle] })));
    }));
};
exports.Switch = Switch;
const getColors = (theme, { disabled, on, pressed }) => {
    const variant = on ? theme.switch.on : theme.switch.off;
    if (disabled) {
        return variant.disabled;
    }
    return pressed ? variant.pressed : variant.default;
};
const getAnimatedSwitchStyles = (theme, animatedColor, animatedPosition, { pressed, on, prevValue, disabled }) => {
    const prevColors = getColors(theme, {
        pressed: false,
        on: prevValue,
        disabled,
    });
    const colors = getColors(theme, {
        pressed,
        on,
        disabled,
    });
    return {
        switchStyle: {
            backgroundColor: animatedColor.current.interpolate({
                inputRange: [0, 1],
                outputRange: [prevColors.fillColor, colors.fillColor],
            }),
            borderColor: animatedColor.current.interpolate({
                inputRange: [0, 1],
                outputRange: [prevColors.borderColor, colors.borderColor],
            }),
        },
        circleStyle: {
            backgroundColor: animatedColor.current.interpolate({
                inputRange: [0, 1],
                outputRange: [prevColors.centerColor, colors.centerColor],
            }),
            marginLeft: animatedPosition.current.interpolate({
                inputRange: [0, 1],
                outputRange: [0, icons_1.IconSize.XSMALL],
            }),
        },
    };
};
