"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTheme = exports.ThemeProvider = void 0;
const react_1 = __importStar(require("react"));
const lodash_1 = require("lodash");
const themes_1 = require("./themes");
const initialState = {
    theme: themes_1.boschTheme,
    setTheme: (_) => {
        /* Initial function */
    },
};
/**
 * Must use arrow function definition, otherwise the generic `T` type
 * cannot be passed. The `once` method calls the method only once.
 */
const createThemeContext = (0, lodash_1.once)(() => react_1.default.createContext(initialState));
const ThemeProvider = (props) => {
    const { initialTheme = initialState.theme, children } = props;
    const [theme, setTheme] = (0, react_1.useState)(initialTheme);
    const ThemeContext = createThemeContext();
    const themeValue = (0, react_1.useMemo)(() => ({ theme, setTheme }), [theme]);
    return (react_1.default.createElement(ThemeContext.Provider, { value: themeValue }, children));
};
exports.ThemeProvider = ThemeProvider;
/**
 * If the `useTheme` is called without a type then the hook will work as *before*.
 * If a type is passed down for the hook, then the hook will return an **object** and a
 * **setter method** with the same types.
 * > OBS: the type must extend the original `Theme` interface.
 * @example
 *
 * const [theme, setTheme] = useTheme<ExtendedTheme>();
 * // set the new extended theme
 * setTheme({ ...boschTheme, highlightText: Colors.BOSCH_RED_50 });
 *
 * // use with confidence
 * const colorOfText: Colors = theme.highlightText;
 */
function useTheme() {
    return (0, react_1.useContext)(createThemeContext());
}
exports.useTheme = useTheme;
