//
//  RNTTPActivityComponentManager.m
//  RNPrivacyComponent
//
//  Created by István Bege on 28/05/2019.
//  Copyright (C) 2019, Bosch Thermotechnik GmbH. All rights reserved.
//
#import <Foundation/Foundation.h>
#import <React/RCTViewManager.h>
#import <React/RCTLog.h>
#import <React/RCTUIManager.h>

#import "RNTTPActivityComponent.h"
#import "ProviderManager.h"
#import "Assembler.h"

@interface RNTTActivityComponentManager : RCTViewManager
@end

@implementation RNTTActivityComponentManager

RCT_EXPORT_MODULE()

RCT_EXPORT_VIEW_PROPERTY(onDocumentLoaded, RCTBubblingEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onScrollEnded, RCTBubblingEventBlock)
RCT_CUSTOM_VIEW_PROPERTY(css, NSString, RNTTPActivityComponent)
{
    [view setCustomCSS:[RCTConvert NSString:json]];
}

RCT_CUSTOM_VIEW_PROPERTY(activityIndicatorStyle, NSString, RNTTPActivityComponent)
{
    NSString* prop = [RCTConvert NSString:json];
    UIActivityIndicatorViewStyle style;
    if ([prop isEqualToString:@"UIActivityIndicatorViewStyleWhiteLarge"]) {
        style = UIActivityIndicatorViewStyleWhiteLarge;
    } else {
        if ([prop isEqualToString:@"UIActivityIndicatorViewStyleWhite"]) {
            style = UIActivityIndicatorViewStyleWhite;
        } else {
            style = UIActivityIndicatorViewStyleGray;
        }
    }
    [view setActivityIndicatorViewStyle:style];
}

- (UIView *)view
{
    return [[RNTTPActivityComponent alloc] init];
}

RCT_EXPORT_METHOD(loadActivityConsent:(NSDictionary*)selectedCountry reactTag:(nonnull NSNumber *)reactTag)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *, RNTTPActivityComponent *> *viewRegistry) {
        RNTTPActivityComponent* view = viewRegistry[reactTag];
        if (![view isKindOfClass:[RNTTPActivityComponent class]]) {
            RCTLogError(@"Invalid view returned from registry, expecting RNTTPActivityComponent, got: %@", view);
        } else {
            TTPCountry* country = [Assembler createCountry:selectedCountry];
            [view loadDocumentWithProvider:[[ProviderManager getInstance] activityProvider] country:[country code] language:[country language] delegate:view];
        }
    }];
}

@end
